@@ -41,7 +41,6 @@ target_requires ${org.macports.fetch} main
 target_prerun ${org.macports.fetch} portfetch::fetch_start
 
 namespace eval portfetch {
-    namespace export suffix
     variable fetch_urls {}
 }
 
@@ -49,7 +48,7 @@ namespace eval portfetch {
 options master_sites patch_sites extract.suffix distfiles patchfiles use_tar \
     use_bzip2 use_lzma use_xz use_zip use_7z use_lzip use_dmg dist_subdir \
     fetch.type fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert \
-    master_sites.mirror_subdir patch_sites.mirror_subdir \
+    fetch.user_agent master_sites.mirror_subdir patch_sites.mirror_subdir \
     bzr.url bzr.revision \
     cvs.module cvs.root cvs.password cvs.date cvs.tag cvs.method \
     svn.url svn.revision svn.method \
@@ -69,30 +68,30 @@ default fetch.type standard
 default bzr.cmd {[findBinary bzr $portutil::autoconf::bzr_path]}
 default bzr.dir {${workpath}}
 default bzr.revision -1
-default bzr.pre_args "--builtin --no-aliases checkout --lightweight --verbose"
-default bzr.args ""
+default bzr.pre_args {--builtin --no-aliases checkout --lightweight --verbose}
+default bzr.args {}
 default bzr.post_args {-r ${bzr.revision} ${bzr.url} ${worksrcdir}}
 
 default cvs.cmd {[findBinary cvs $portutil::autoconf::cvs_path]}
-default cvs.password ""
+default cvs.password {}
 default cvs.dir {${workpath}}
-default cvs.method {export}
+default cvs.method export
 default cvs.module {$distname}
-default cvs.tag ""
-default cvs.date ""
+default cvs.tag {}
+default cvs.date {}
 default cvs.env {CVS_PASSFILE=${workpath}/.cvspass}
 default cvs.pre_args {-z9 -f -d ${cvs.root}}
-default cvs.args ""
+default cvs.args {}
 default cvs.post_args {${cvs.module}}
 
-default svn.cmd {[portfetch::find_svn_path]}
+default svn.cmd {${prefix_frozen}/bin/svn}
 default svn.dir {${workpath}}
-default svn.method {export}
-default svn.revision ""
+default svn.method export
+default svn.revision {}
 default svn.env {}
-default svn.pre_args "--non-interactive --trust-server-cert"
-default svn.args ""
-default svn.post_args ""
+default svn.pre_args --non-interactive
+default svn.args {}
+default svn.post_args {}
 
 default git.cmd {[portfetch::find_git_path]}
 default git.dir {${workpath}}
@@ -103,18 +102,19 @@ default hg.dir {${workpath}}
 default hg.tag tip
 
 # Set distfiles
-default distfiles {[portfetch::suffix $distname]}
+default distfiles {[list [portfetch::suffix [join $distname]]]}
 default dist_subdir {${name}}
 
 # user name & password
-default fetch.user ""
-default fetch.password ""
+default fetch.user {}
+default fetch.password {}
 # Use EPSV for FTP transfers
 default fetch.use_epsv yes
 # Ignore SSL certificate
 default fetch.ignore_sslcert no
 # Use remote timestamps
 default fetch.remote_time no
+default fetch.user_agent {}
 
 default global_mirror_site macports_distfiles
 default mirror_sites.listfile mirror_sites.tcl
@@ -186,19 +186,14 @@ proc portfetch::set_fetch_type {option action args} {
                 depends_fetch-append bin:cvs:cvs
             }
             svn {
-                # Sierra is the first macOS version whose svn supports modern TLS cipher suites.
-                if {${os.major} >= 16 || ${os.platform} ne "darwin"} {
-                    depends_fetch-append bin:svn:subversion
-                } else {
-                    depends_fetch-append port:subversion
-                }
+                depends_fetch-append path:bin/svn:subversion
             }
             git {
-                # Mavericks is the first OS X version whose git supports modern TLS cipher suites.
-                if {${os.major} >= 13 || ${os.platform} ne "darwin"} {
+                # Oldest macOS version whose git can validate GitHub's SSL certificate.
+                if {${os.major} >= 14 || ${os.platform} ne "darwin"} {
                     depends_fetch-append bin:git:git
                 } else {
-                    depends_fetch-append port:git
+                    depends_fetch-append path:bin/git:git
                 }
             }
             hg {
@@ -208,23 +203,13 @@ proc portfetch::set_fetch_type {option action args} {
     }
 }
 
-proc portfetch::find_svn_path {args} {
-    global prefix os.platform os.major
-    # Sierra is the first macOS version whose svn supports modern TLS cipher suites.
-    if {${os.major} >= 16 || ${os.platform} ne "darwin"} {
-        return [findBinary svn $portutil::autoconf::svn_path]
-    } else {
-        return ${prefix}/bin/svn
-    }
-}
-
 proc portfetch::find_git_path {args} {
-    global prefix os.platform os.major
-    # Mavericks is the first OS X version whose git supports modern TLS cipher suites.
-    if {${os.major} >= 13 || ${os.platform} ne "darwin"} {
+    global prefix_frozen os.platform os.major
+    # Oldest macOS version whose git can validate GitHub's SSL certificate.
+    if {${os.major} >= 14 || ${os.platform} ne "darwin"} {
         return [findBinary git $portutil::autoconf::git_path]
     } else {
-        return ${prefix}/bin/git
+        return ${prefix_frozen}/bin/git
     }
 }
 
@@ -234,11 +219,8 @@ set_ui_prefix
 # Given a distname, return the distname with extract.suffix appended
 proc portfetch::suffix {distname} {
     global extract.suffix
-    return "${distname}${extract.suffix}"
+    return ${distname}[join ${extract.suffix}]
 }
-# XXX import suffix into the global namespace as it is currently used from
-# Portfiles, but should better go somewhere else
-namespace import portfetch::suffix
 
 # Checks patch files and their tags to assemble url lists for later fetching
 proc portfetch::checkpatchfiles {urls} {
@@ -296,8 +278,8 @@ proc portfetch::checkfiles {urls} {
     global global_mirror_site ports_fetch_no-mirrors license
     upvar $urls fetch_urls
 
-    set sites [list patch_sites {} \
-                    master_sites {}]
+    set sites [list patch_sites [list] \
+                    master_sites [list]]
     if {(![info exists ports_fetch_no-mirrors] || ${ports_fetch_no-mirrors} eq "no") \
             && [lsearch -exact -nocase $license "nomirror"] == -1} {
         set sites [list patch_sites [list $global_mirror_site PATCH_SITE_LOCAL] \
@@ -355,7 +337,7 @@ proc portfetch::bzrfetch {args} {
 # Perform a CVS login and fetch, storing the CVS login
 # information in a custom .cvspass file
 proc portfetch::cvsfetch {args} {
-    global workpath cvs.env cvs.cmd cvs.args cvs.post_args \
+    global cvs.cmd cvs.args cvs.post_args \
            cvs.root cvs.date cvs.tag cvs.method cvs.password
            patch_sites patchfiles filespath
 
@@ -378,7 +360,7 @@ proc portfetch::cvsfetch {args} {
         set cvs.cmd "echo ${cvs.password} | ${cvs.cmd}"
         set cvs.args login
         set cvs.post_args ""
-        if {[catch {command_exec cvs -notty "" "2>&1"} result]} {
+        if {[catch {command_exec -notty cvs "" "2>&1"} result]} {
             return -code error [msgcat::mc "CVS login failed"]
         }
         set cvs.cmd ${savecmd}
@@ -435,6 +417,10 @@ proc portfetch::svnfetch {args} {
         append svn.url "@${svn.revision}"
     }
 
+    if {[option fetch.ignore_sslcert]} {
+        svn.pre_args-append --trust-server-cert
+    }
+
     set proxy_args [svn_proxy_args ${svn.url}]
 
     set svn.args "${svn.method} ${svn.args} ${proxy_args} ${svn.url}"
@@ -453,21 +439,21 @@ proc portfetch::svnfetch {args} {
 # Perform a git fetch
 proc portfetch::gitfetch {args} {
     global worksrcpath patchfiles \
-           git.url git.branch git.sha1 git.cmd
+           git.url git.branch git.cmd
 
     set options "--progress"
     if {${git.branch} eq ""} {
         # if we're just using HEAD, we can make a shallow repo
         append options " --depth=1"
     }
-    set cmdstring "${git.cmd} clone $options ${git.url} ${worksrcpath} 2>&1"
+    set cmdstring "${git.cmd} clone $options ${git.url} [shellescape ${worksrcpath}] 2>&1"
     ui_debug "Executing: $cmdstring"
     if {[catch {system $cmdstring} result]} {
         return -code error [msgcat::mc "Git clone failed"]
     }
 
     if {${git.branch} ne ""} {
-        set env "GIT_DIR=${worksrcpath}/.git GIT_WORK_TREE=${worksrcpath}"
+        set env "GIT_DIR=[shellescape ${worksrcpath}/.git] GIT_WORK_TREE=[shellescape ${worksrcpath}]"
         set cmdstring "$env ${git.cmd} checkout -q ${git.branch} 2>&1"
         ui_debug "Executing $cmdstring"
         if {[catch {system $cmdstring} result]} {
@@ -484,7 +470,7 @@ proc portfetch::gitfetch {args} {
 
 # Perform a mercurial fetch.
 proc portfetch::hgfetch {args} {
-    global worksrcpath prefix_frozen patchfiles hg.url hg.tag hg.cmd \
+    global worksrcpath patchfiles hg.url hg.tag hg.cmd \
            fetch.ignore_sslcert
 
     set insecureflag ""
@@ -492,7 +478,7 @@ proc portfetch::hgfetch {args} {
         set insecureflag " --insecure"
     }
 
-    set cmdstring "${hg.cmd} clone${insecureflag} --rev \"${hg.tag}\" ${hg.url} ${worksrcpath} 2>&1"
+    set cmdstring "${hg.cmd} clone${insecureflag} --rev \"${hg.tag}\" ${hg.url} [shellescape ${worksrcpath}] 2>&1"
     ui_debug "Executing: $cmdstring"
     if {[catch {system $cmdstring} result]} {
         return -code error [msgcat::mc "Mercurial clone failed"]
@@ -508,13 +494,13 @@ proc portfetch::hgfetch {args} {
 # Perform a standard fetch, assembling fetch urls from
 # the listed url variable and associated distfile
 proc portfetch::fetchfiles {args} {
-    global distpath all_dist_files UI_PREFIX \
+    global distpath UI_PREFIX \
            fetch.user fetch.password fetch.use_epsv fetch.ignore_sslcert fetch.remote_time \
-           portverbose
+           fetch.user_agent portverbose
     variable fetch_urls
     variable urlmap
 
-    set fetch_options {}
+    set fetch_options [list]
     if {[string length ${fetch.user}] || [string length ${fetch.password}]} {
         lappend fetch_options -u
         lappend fetch_options "${fetch.user}:${fetch.password}"
@@ -528,10 +514,14 @@ proc portfetch::fetchfiles {args} {
     if {${fetch.remote_time} ne "no"} {
         lappend fetch_options "--remote-time"
     }
+    if {${fetch.user_agent} ne ""} {
+        lappend fetch_options "--user-agent"
+        lappend fetch_options "${fetch.user_agent}"
+    }
     if {$portverbose eq "yes"} {
         lappend fetch_options "--progress"
         lappend fetch_options "builtin"
-    } elseif {[llength [info commands ui_progress_download]] > 0} {
+    } else {
         lappend fetch_options "--progress"
         lappend fetch_options "ui_progress_download"
     }
@@ -556,12 +546,12 @@ proc portfetch::fetchfiles {args} {
             foreach site $urlmap($url_var) {
                 ui_notice "$UI_PREFIX [format [msgcat::mc "Attempting to fetch %s from %s"] $distfile $site]"
                 set file_url [portfetch::assemble_url $site $distfile]
-                try -pass_signal {
+                macports_try -pass_signal {
                     curl fetch {*}$fetch_options $file_url "${distpath}/${distfile}.TMP"
                     file rename -force "${distpath}/${distfile}.TMP" "${distpath}/${distfile}"
                     set fetched 1
                     break
-                } catch {{*} eCode eMessage} {
+                } on error {eMessage} {
                     ui_debug [msgcat::mc "Fetching distfile failed: %s" $eMessage]
                     set lastError $eMessage
                 } finally {
@@ -606,7 +596,9 @@ proc portfetch::fetch_addfilestomap {filemapname} {
 proc portfetch::fetch_init {args} {
     variable fetch_urls
 
-    portfetch::checkfiles fetch_urls
+    if {${fetch_urls} eq ""} {
+        portfetch::checkfiles fetch_urls
+    }
 }
 
 proc portfetch::fetch_start {args} {
